@@ -13,7 +13,7 @@ from typing import Optional, Dict, Any, Tuple
 import dspy
 from pathlib import Path
 
-def focus_vscode_workspace(workspace_env_var: str, verbose: bool = True) -> bool:
+def focus_vscode_workspace(workspace_env_var: str, verbose: bool = False) -> bool:
     """Focus the VS Code workspace."""
     if not workspace_env_var:
         return False
@@ -24,7 +24,7 @@ def focus_vscode_workspace(workspace_env_var: str, verbose: bool = True) -> bool
         return False
     try:
         from .open_vscode_workspace import open_and_focus_workspace
-        success = open_and_focus_workspace(workspace_path, focus=True)
+        success = open_and_focus_workspace(workspace_path, focus=True, verbose=verbose)
         if success and verbose:
             print("  VS Code workspace focused successfully.")
         return success
@@ -134,11 +134,12 @@ class VSCodeCopilot(dspy.BaseLM):
     Uses session-based temporary files to avoid race conditions between tests.
     """
     
-    def __init__(self, workspace_path: str, workspace_env_var: str, polling_timeout: int = 120, **kwargs):
+    def __init__(self, workspace_path: str, workspace_env_var: str, polling_timeout: int = 120, verbose: bool = False, **kwargs):
         super().__init__(model="vscode-copilot", **kwargs)
         self.workspace_path = workspace_path
         self.workspace_env_var = workspace_env_var
         self.polling_timeout = polling_timeout
+        self.verbose = verbose
         # Generate a unique session ID for this model instance
         self.session_id = str(uuid.uuid4())[:8]
         
@@ -163,27 +164,29 @@ class VSCodeCopilot(dspy.BaseLM):
         file_list = []
         token_list = []
         for i, instruction_file in enumerate(instruction_files, 1):
-            file_name = instruction_file.split('/')[-1]  # Get just the filename
-            file_list.append(f"`#file:{instruction_file}`")
-            token_list.append(f"INSTRUCTIONS_READ: [{file_name}](http://_vscodecontentref_/{i}) SHA256=<hex>")
+            # instruction_file is already an absolute path from yaml_parser
+            abs_path = Path(instruction_file)
+            file_name = abs_path.name  # Get just the filename
+            file_uri = abs_path.as_uri()
+            file_list.append(f"[{file_name}]({file_uri})")
+            token_list.append(f"INSTRUCTIONS_READ: `{file_name}` i={i} SHA256=<hex>")
         
         # Create single consolidated pre-read instruction
         files_text = ", ".join(file_list)
         tokens_text = "\n".join(token_list)
         
         consolidated_instruction = (
-            f"**MANDATORY STEP — PRE-READ ALL INSTRUCTIONS**: Before any analysis, open and read all instruction files: {files_text}. "
-            f"Locate each file using its filename or an explicit filesystem path (do not rely on a workspace text search). "
+            f"Read all instruction files: {files_text}. "
             f"After reading each file, compute its SHA256 hash using this PowerShell command: "
             f"`Get-FileHash -Algorithm SHA256 -LiteralPath '<file-path>' | Select-Object -ExpandProperty Hash`. "
             f"Then include, at the top of your reply, these exact tokens on separate lines:\n\n"
             f"{tokens_text}\n\n"
             f"Replace `<hex>` with the actual SHA256 hash value computed from the PowerShell command. "
-            f"Follow each token with a one-paragraph summary of the specific rules applied from that file. "
-            f"If any file is missing, fail with ERROR: missing-file <filename> and stop."
+            f"If any file is missing, fail with ERROR: missing-file <filename> and stop.\n\n"
+            f"Then fetch all documentation required by the instructions before proceeding with your task."
         )
         
-        return f"## 1. Mandatory Pre-Read\n\n{consolidated_instruction}\n\n"
+        return f"[[ ## mandatory_pre_read ## ]]\n\n{consolidated_instruction}\n\n"
     
     def _prepare_session_files(self, test_case_id: str) -> Tuple[Path, Path, Path, Path]:
         """
@@ -241,16 +244,27 @@ class VSCodeCopilot(dspy.BaseLM):
         import time
         
         try:
-            # Use PowerShell to run VS Code chat command with the request file
-            powershell_command = f'Get-Content -Raw -LiteralPath "{request_file_path.resolve()}"'
-            vscode_command = f'code chat -r "run command <powershell>{powershell_command}</powershell> and follow its instructions."'
-            
-            # Execute the PowerShell command
-            cmd = ['pwsh', '-NoProfile', '-NonInteractive', '-Command', vscode_command]
+            # Build the VS Code CLI chat command directly (avoid dependency on pwsh)
+            # We still embed a PowerShell snippet inside the Copilot chat instruction so Copilot
+            # can read the request file contents within VS Code context.
+            inner_powershell_command = f'Get-Content -Raw -LiteralPath "{request_file_path.resolve()}"'
+            chat_instruction = f"run command <powershell>{inner_powershell_command}</powershell> and follow its instructions."
+
+            # Prefer calling the VS Code CLI directly. This removes requirement for 'pwsh'.
+            # The 'code' executable must be on PATH (VS Code's 'Shell Command: Install 'code'' setting on macOS,
+            # or automatically available on Windows after install).
+            import shutil
+            code_cli = shutil.which('code')
+            if not code_cli:
+                return ("Error: VS Code CLI 'code' was not found on PATH. "
+                        "Ensure VS Code is installed and the command line launcher is enabled. "
+                        "On Windows this is usually automatic; on macOS use the Command Palette: 'Shell Command: Install code command'.")
+
+            cmd = [code_cli, 'chat', '-r', chat_instruction]
             
             # Also print a concise summary to stdout
             try:
-                print(f"PowerShell + VS Code: {test_case_id}.req.md → {reply_final_path.name} (session: {self.session_id})")
+                print(f"  PowerShell + VS Code: {test_case_id}.req.md → {reply_final_path.name} (session: {self.session_id})")
             except Exception:
                 pass
 
@@ -266,12 +280,13 @@ class VSCodeCopilot(dspy.BaseLM):
             if result.returncode != 0:
                 # Build a richer error including command details for troubleshooting
                 try:
-                    (session_dir / 'last_powershell_stderr.log').write_text(result.stderr or '', encoding='utf-8')
+                    (session_dir / 'last_cli_stderr.log').write_text(result.stderr or '', encoding='utf-8')
                 except Exception:
                     pass
                 error_msg = (
-                    f"PowerShell command failed (exit code {result.returncode}): {result.stderr}\n"
-                    f"Command: {vscode_command}\n"
+                    f"VS Code CLI chat command failed (exit code {result.returncode}).\n"
+                    f"Command: {' '.join(cmd)}\n"
+                    f"Stderr: {result.stderr}\n"
                     f"Request file: {request_file_path.resolve()}\n"
                     f"Note: Full command details saved under {session_dir}"
                 )
@@ -302,13 +317,13 @@ class VSCodeCopilot(dspy.BaseLM):
             # Re-raise timeout errors to be caught by retry logic
             raise
         except FileNotFoundError:
-            return "Error: PowerShell not found. Please ensure 'pwsh' is available in PATH"
+            return "Error: Failed to execute VS Code CLI. Confirm that 'code' is available on PATH."
         except Exception as e:
             return f"Error: {str(e)}"
 
     def forward(self, prompt: str = None, messages=None, test_case_id: str = None, instruction_files: list = None, task: str = None, **kwargs):
         """Create a request file and execute VS Code with PowerShell command to read it."""
-        focus_vscode_workspace(self.workspace_env_var, verbose=True)
+        focus_vscode_workspace(self.workspace_env_var, verbose=self.verbose)
         
         from types import SimpleNamespace
 
@@ -324,7 +339,7 @@ class VSCodeCopilot(dspy.BaseLM):
                 final_prompt += self._build_mandatory_preread_block(instruction_files)
             
             # Section 2: Add the task with clear header
-            final_prompt += "## 2. Task\n\n"
+            final_prompt += "[[ ## task ## ]]\n\n"
             final_prompt += task + "\n\n"
             
             # Use the clean final prompt instead of the complex structure
@@ -336,7 +351,7 @@ class VSCodeCopilot(dspy.BaseLM):
         # Create the request file content with file handling instructions
         enhanced_prompt = (
             f"{(actual_prompt or '').strip()}\n\n"
-            f"## 3. Output Instructions\n\n"
+            f"[[ ## output_instructions ## ]]\n\n"
             f"**IMPORTANT**: Follow these exact steps:\n"
             f"1. Write your complete response to: {reply_tmp_path.resolve()}\n"
             f"2. When completely finished, run this PowerShell command to signal completion:\n"
@@ -350,6 +365,21 @@ class VSCodeCopilot(dspy.BaseLM):
         except Exception as e:
             raise RuntimeError(f"Failed to write request file {request_file_path}: {e}")
 
+        # Record raw request metadata for downstream capture in evaluation results
+        try:
+            self._last_raw_request = {
+                'provider': 'vscode',
+                'test_case_id': test_case_id,
+                'task': task,
+                'instruction_files': instruction_files,
+                'request_file': str(request_file_path.resolve()),
+                'reply_tmp_file': str(reply_tmp_path.resolve()),
+                'reply_final_file': str(reply_final_path.resolve()),
+                'enhanced_prompt': enhanced_prompt
+            }
+        except Exception:
+            pass
+
         # Execute the VS Code command and poll for response
         response_content = self._execute_vscode_command_and_poll(
             request_file_path, reply_final_path, reply_tmp_path, session_dir, test_case_id
@@ -393,7 +423,8 @@ class VSCodeCopilot(dspy.BaseLM):
             return prompt
         
         if messages:
-            # Attempt to extract only the 'task' block from DSPy's structured messages
+            # Extract task content from DSPy's structured messages
+            # Look for the request field used by QuerySignature
             try:
                 combined = ''
                 if isinstance(messages, list):
@@ -405,13 +436,17 @@ class VSCodeCopilot(dspy.BaseLM):
                 else:
                     combined = str(messages)
                 import re as _re
-                # Find all task blocks
-                matches = list(_re.finditer(r"\[\[\s*##\s*task\s*##\s*\]\]\s*(.*?)\s*(?=\[\[|$)", combined, flags=_re.IGNORECASE | _re.DOTALL))
-                # Prefer the last filled block that does not contain the placeholder '{task}'
+                
+                # Look for request field (QuerySignature)
+                # Note: We don't include 'expected_outcome' or 'outcome' to avoid leaking the answer
+                pattern = r"\[\[\s*##\s*request\s*##\s*\]\]\s*(.*?)\s*(?=\[\[|$)"
+                matches = list(_re.finditer(pattern, combined, flags=_re.IGNORECASE | _re.DOTALL))
+                # Prefer the last filled block that does not contain placeholders
                 for m in reversed(matches):
                     candidate = m.group(1).strip()
-                    if candidate and '{task}' not in candidate:
+                    if candidate and '{request}' not in candidate:
                         return candidate
+                
                 # Heuristic fallback: pick the first question-like line
                 for line in combined.splitlines():
                     line_s = line.strip()
@@ -442,13 +477,16 @@ class VSCodeCopilot(dspy.BaseLM):
         # For VSCodeCopilot, we need to call the model directly with test_case_id
         # For VSCodeCopilot, use a simple prompt structure since mandatory preread handles instructions
         # Just pass the task directly, don't build complex prompt sections
-        prompt = kwargs.get('task', '')
+        
+        # Get task content from the request field (QuerySignature)
+        # Note: We don't use 'expected_outcome' or 'outcome' to avoid leaking the answer
+        task_content = kwargs.get('request', '')
         
         # Get instruction files from guideline_paths
         instruction_files = kwargs.get('guideline_paths', [])
         
         # Call the VSCodeCopilot model directly with test_case_id, instruction files, and task
-        response = self.forward(prompt=prompt, test_case_id=test_case_id, instruction_files=instruction_files, task=kwargs.get('task'))
+        response = self.forward(prompt=task_content, test_case_id=test_case_id, instruction_files=instruction_files, task=task_content)
         
         # Extract the answer from the response
         import json
@@ -467,7 +505,7 @@ class VSCodeCopilot(dspy.BaseLM):
         result.answer = answer_content
         return result
 
-def create_model(provider: str, model: str, settings: Dict[str, Any] = None, **kwargs):
+def create_model(provider: str, model: str, settings: Dict[str, Any] = None, verbose: bool = False, **kwargs):
     """
     Factory function to create model instances based on provider.
     
@@ -475,6 +513,7 @@ def create_model(provider: str, model: str, settings: Dict[str, Any] = None, **k
         provider: 'anthropic', 'azure', 'vscode', or 'mock'
         model: Model name/deployment
         settings: Provider-specific settings from targets.yaml
+        verbose: Whether to print verbose output
         **kwargs: Additional model configuration
     
     Returns:
@@ -483,6 +522,9 @@ def create_model(provider: str, model: str, settings: Dict[str, Any] = None, **k
     provider = provider.lower()
     settings = settings or {}
     
+    # Extract optional cache control
+    disable_cache: bool = bool(kwargs.pop('disable_cache', False))
+
     if provider == "vscode":
         # Get environment variable name from target settings
         workspace_env_var = settings.get('workspace_env_var')
@@ -496,7 +538,7 @@ def create_model(provider: str, model: str, settings: Dict[str, Any] = None, **k
         
         # Extract polling timeout from kwargs, default to 120 seconds
         polling_timeout = kwargs.pop('polling_timeout', 120)
-        return VSCodeCopilot(workspace_path=workspace_path, workspace_env_var=workspace_env_var, polling_timeout=polling_timeout, **kwargs)
+        return VSCodeCopilot(workspace_path=workspace_path, workspace_env_var=workspace_env_var, polling_timeout=polling_timeout, verbose=verbose, **kwargs)
     elif provider == "anthropic":
         # Get environment variable names from target settings
         api_key_var = settings.get('api_key')
@@ -510,7 +552,10 @@ def create_model(provider: str, model: str, settings: Dict[str, Any] = None, **k
         if not api_key:
             raise ValueError(f"Environment variable '{api_key_var}' is not set.")
         
-        lm = dspy.LM(f"anthropic/{model}", api_key=api_key)
+        if disable_cache:
+            lm = dspy.LM(f"anthropic/{model}", api_key=api_key, cache=False)
+        else:
+            lm = dspy.LM(f"anthropic/{model}", api_key=api_key)
         return StandardLM(lm)
     elif provider == "azure":
         # Get environment variable names from target settings
@@ -533,16 +578,22 @@ def create_model(provider: str, model: str, settings: Dict[str, Any] = None, **k
         is_reasoning_model = any(reasoning_model in model.lower() for reasoning_model in reasoning_models)
         
         if is_reasoning_model:
-            lm = dspy.LM(f"azure/{model}", api_key=api_key, api_base=endpoint, temperature=1.0, max_tokens=16000)
+            if disable_cache:
+                lm = dspy.LM(f"azure/{model}", api_key=api_key, api_base=endpoint, temperature=1.0, max_tokens=16000, cache=False)
+            else:
+                lm = dspy.LM(f"azure/{model}", api_key=api_key, api_base=endpoint, temperature=1.0, max_tokens=16000)
         else:
-            lm = dspy.LM(f"azure/{model}", api_key=api_key, api_base=endpoint)
+            if disable_cache:
+                lm = dspy.LM(f"azure/{model}", api_key=api_key, api_base=endpoint, cache=False)
+            else:
+                lm = dspy.LM(f"azure/{model}", api_key=api_key, api_base=endpoint)
         return StandardLM(lm)
     elif provider == "mock":
         return MockModel(**kwargs)
     else:
         raise ValueError(f"Unsupported provider: {provider}. Supported: anthropic, azure, vscode, mock")
 
-def configure_dspy_model(provider: str, model: str, settings: Dict[str, Any] = None, **kwargs):
+def configure_dspy_model(provider: str, model: str, settings: Dict[str, Any] = None, verbose: bool = False, **kwargs):
     """
     Configure DSPy with the specified model.
     
@@ -550,7 +601,8 @@ def configure_dspy_model(provider: str, model: str, settings: Dict[str, Any] = N
         provider: Model provider
         model: Model name
         settings: Provider-specific settings
+        verbose: Whether to print verbose output
         **kwargs: Additional configuration
     """
-    model_instance = create_model(provider, model, settings, **kwargs)
+    model_instance = create_model(provider, model, settings, verbose=verbose, **kwargs)
     dspy.settings.configure(lm=model_instance)
